%
% This is $Id: mpbugs.ch,v 1.7 2005/03/18 19:45:56 taco Exp $ 
%
% This change file is a transient source file that will always contain
% the set of bugs in mp.web that will be merged in for the next public 
% release of MetaPost.
%
% This file is internal to the MetaPost project at Sarovar. If you 
% discover this file outside of a 'beta testing' environment, something 
% has probably gone wrong, and it would be wise to check the project's 
% website at http://metapost.sarovar.org
%
% The code comprising this file is released to the public domain by the 
% various authors that contributed to the code below.
%
% There is absolutely NO WARRENTEE OF ANY KIND with respect to this
% software.  The entire risk as to the quality and performance of this
% software is with the user.

@x l. 3192
@ And now let's complete our collection of numeric utility routines
by considering random number generation.
\MP\ generates pseudo-random numbers with the additive scheme recommended
in Section 3.6 of {\sl The Art of Computer Programming}; however, the
results are random fractions between 0 and |fraction_one-1|, inclusive.

There's an auxiliary array |randoms| that contains 55 pseudo-random
fractions. Using the recurrence $x_n=(x_{n-55}-x_{n-31})\bmod 2^{28}$,
we generate batches of 55 new $x_n$'s at a time by calling |new_randoms|.
The global variable |j_random| tells which element has most recently
been consumed.

@<Glob...@>=
@!randoms:array[0..54] of fraction; {the last 55 random values generated}
@!j_random:0..54; {the number of unused |randoms|}
@y
@ And now let's complete our collection of numeric utility routines
by considering random number generation.
\MP\ generates pseudo-random numbers with the additive scheme recommended
in Section 3.6 of {\sl The Art of Computer Programming}; however, the
results are random fractions between 0 and |fraction_one-1|, inclusive.

There's an auxiliary array |randoms| that contains 55 pseudo-random
fractions. Using the recurrence $x_n=(x_{n-55}-x_{n-31})\bmod 2^{28}$,
we generate batches of 55 new $x_n$'s at a time by calling |new_randoms|.
The global variable |j_random| tells which element has most recently
been consumed.
The global variable |sys_random_seed| was introduced in version 0.9,
for the sole reason of stressing the fact that the initial value of the
random seed is system-dependant. The pascal code below will initialize 
this variable to |(internal[time] div unity)+internal[day]|, but this is
not good enough on modern fast machines that are capable of running
multiple MetaPost processes within the same second.
@^system dependencies@>

@<Glob...@>=
@!randoms:array[0..54] of fraction; {the last 55 random values generated}
@!j_random:0..54; {the number of unused |randoms|}
@!sys_random_seed:scaled; {the default random seed}
@z

@x [453] l.8993 Fix filldraw bounding box 
@ @<Other cases for updating the bounding box...@>=
fill_code: begin path_bbox(path_p(p));
  adjust_bbox(h);
  end;
@y
@ @<Other cases for updating the bounding box...@>=
fill_code: begin path_bbox(path_p(p));
   if pen_p(p)<>null then
     begin x0:=minx; y0:=miny;
     x1:=maxx; y1:=maxy;
     pen_bbox(pen_p(p));
     minx:=minx+x0;
     miny:=miny+y0;
     maxx:=maxx+x1;
     maxy:=maxy+y1;
     end;
  adjust_bbox(h);
  end;
@z


@x [468] l.9161 Fix 'dead path' memory consumption
@ We must be careful not to remove the only cubic in a cycle.

@<Advance |p| to node |q|, removing any ``dead'' cubics...@>=
repeat r:=link(p);
if x_coord(p)=right_x(p) then if y_coord(p)=right_y(p) then
 if x_coord(p)=left_x(r) then if y_coord(p)=left_y(r) then
  if x_coord(p)=x_coord(r) then if y_coord(p)=y_coord(r) then
    if r<>p then
      @<Remove the cubic following |p| and update the data structures
        to merge |r| into |p|@>;
p:=r;
until p=q
@y
@ We must be careful not to remove the only cubic in a cycle.

But we must also be careful for another reason. If the user-supplied 
path starts with a set of deferate cubics, these should not be removed
because at this point we cannot do so cleanly. The relevant bug is
tracker id 267, bugs 52c, reported by Boguslav. 

@<Advance |p| to node |q|, removing any ``dead'' cubics...@>=
repeat r:=link(p);
if x_coord(p)=right_x(p) then if y_coord(p)=right_y(p) then
 if x_coord(p)=left_x(r) then if y_coord(p)=left_y(r) then
  if x_coord(p)=x_coord(r) then if y_coord(p)=y_coord(r) then
    if r<>p then if r<>q then
      @<Remove the cubic following |p| and update the data structures
        to merge |r| into |p|@>;
p:=r;
until p=q
@z


@x [539] l.10508 Serial number wrapping
@d s_scale=64 {the serial numbers are multiplied by this factor}
@d new_indep(#)== {create a new independent variable}
  begin type(#):=independent; serial_no:=serial_no+s_scale;
  value(#):=serial_no;
  end
@y

The next patch detects overflow of independent-variable serial 
numbers. Diagnosed and patched by Thorsten Dahlheimer.

@d s_scale=64 {the serial numbers are multiplied by this factor}
@d max_indep_vars==@'177777777 {$2^{25}-1$}
@d max_serial_no==@'17777777700 {|max_indep_vars*s_scale|}
@d new_indep(#)== {create a new independent variable}
  begin if serial_no=max_serial_no then
    overflow("independent variables",max_indep_vars);
  type(#):=independent; serial_no:=serial_no+s_scale;
  value(#):=serial_no;
  end
@z


@x l. 16878
turning_op:if cur_type=pair_type then flush_cur_exp(0)
  else if cur_type<>path_type then bad_unary(turning_op)
  else if left_type(cur_exp)=endpoint then
     flush_cur_exp(0) {not a cyclic path}
  else  begin cur_exp:=offset_prep(cur_exp,test_pen);
    if internal[tracing_specs]>unity then
      print_spec(cur_exp,test_pen," (for turningnumber)");
    flush_cur_exp(count_turns(cur_exp));
    end;
@y
turning_op:if cur_type=pair_type then flush_cur_exp(0)
  else if cur_type<>path_type then bad_unary(turning_op)
  else if left_type(cur_exp)=endpoint then
     flush_cur_exp(0) {not a cyclic path}
  else  begin 
    flush_cur_exp(turn_cycles(cur_exp));
    end;


@ This code is based on Bogus\l{}av Jackowski's
|emergency_turningnumber| macro, with some minor changes by Taco
Hoekwater. The macro code looked more like this:
{\obeylines
vardef turning\_number primary p =
~~save res, ang, turns;
~~res := 0;
~~if length p <= 2: 
~~~~if Angle ((point 0 of p) - (postcontrol 0 of p)) >= 0:  1  else: -1 fi
~~else:
~~~~for t = 0 upto length p-1 :
~~~~~~angc := Angle ((point t+1 of p)  - (point t of p)) 
~~~~~~~~- Angle ((point t of p) - (point t-1 of p));
~~~~~~if angc > 180: angc := angc - 360; fi;
~~~~~~if angc < -180: angc := angc + 360; fi;
~~~~~~res  := res + angc;
~~~~endfor;
~~res/360
~~fi
enddef;}
The general idea is to calculate only the sum of the angles of straight lines between
the points, of a path, not worrying about cusps or self-intersections in the segments
at all. If the segment is not well-behaved, the result is not necesarily correct. But
the old code was not always correct either, and worse, it sometimes failed for well-behaved
paths as well. All known bugs that were triggered by the original code no longer occur
with this code, and it runs roughly 3 times as fast because the algorithm is much simpler.

@ The macro |Angle()| returns the value of the |angle| primitive, or $0$ if the argument is
|origin|. Converting that calling convention to web code gives the |an_angle| function.

@<Declare unary action...@>=
function an_angle (@!xpar,@!ypar:scaled):angle;
begin
  if (not ((xpar=0) and (ypar=0))) then
    an_angle := n_arg(xpar,ypar)
  else
    an_angle := 0;
end;


@ It is possible to overflow the return value of the |turn_cycles|
function when the path is sufficiently long and winding, but I am not
going to bother testing for that. In any case, it would only return
the looped result value, which is not a big problem.

The macro code for the repeat loop was a bit nicer to look
at than the pascal code, because it could use |point -1 of p|. In
pascal, the fastest way to loop around the path is not to look
backward once, but forward twice. These defines help hide the trick.

@d p_to==link(link(p))
@d p_here==link(p)
@d p_from==p

@<Declare unary action...@>=
function turn_cycles (@!c:pointer):scaled;
var @!res,ang:angle; { the angles of intermediate results }
@!turns:scaled;  { the turn counter }
@!p:pointer;     { for running around the path }
begin  res:=0;  turns:= 0; p:=c;
if ((link(p) = p) or (link(link(p)) = p)) then
  if an_angle (x_coord(p) - right_x(p),  y_coord(p) - right_y(p)) >= 0 then
     turn_cycles := unity
  else
     turn_cycles := -unity
else begin
  repeat 
    ang  := an_angle (x_coord(p_to) - x_coord(p_here), y_coord(p_to) - y_coord(p_here)) 
      	- an_angle (x_coord(p_here) - x_coord(p_from), y_coord(p_here) - y_coord(p_from));
    reduce_angle(ang);
    res  := res + ang;
    if res >= three_sixty_deg then  begin 
      res := res - three_sixty_deg; 
      turns := turns + unity;
    end;
    if res <= -three_sixty_deg then begin 
      res := res + three_sixty_deg; 
      turns := turns - unity; 
    end;
    p := link(p);
  until p=c;
  turn_cycles := turns;
end;
end;
@z

@x l. 22145
if internal[prologues]>0 then @<Print the prologue@>;
@y
print("%%BeginProlog"); print_ln;
if internal[prologues]>0 then @<Print the prologue@>;
@z

@x
fix_date_and_time; init_randoms((internal[time] div unity)+internal[day]);@/
@y
fix_date_and_time;@/
sys_random_seed := (internal[time] div unity)+internal[day];@/
init_randoms(sys_random_seed);@/
@z